
__version__ = "$Id: constants.py 56 2018-09-14 16:42:15Z lbusoni $"


class Constants:
    PICKLE_PROTOCOL= 2
    APP_NAME= "inaf.arcetri.ao.plico"
    APP_AUTHOR= "INAF Arcetri Adaptive Optics"

    PORT_REPLY_OFFSET= 0
    PORT_STATUS_OFFSET= 1
    PORT_PUBLISHER_OFFSET= 2
    PORT_DISPLAY_OFFSET= 3
